package org.cyrano.editor.listeners;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.EventListenerList;

import org.cyrano.utils.ListUtils;

public class UndoRedoManager {

  private static final UndoRedoManager instance = //
  new UndoRedoManager();

  // --------------------------------------------------------------------------------

  public static UndoRedoManager getInstance() {
    return instance;
  }

  // --------------------------------------------------------------------------------

  private EventListenerList eventListenerList = //
  new EventListenerList();

  private CommandListenerProxy commandListenerProxy = //
  new CommandListenerProxy(eventListenerList);

  private List<Command> undoList = new ArrayList<Command>();
  private List<Command> redoList = new ArrayList<Command>();

  // --------------------------------------------------------------------------------

  private UndoRedoManager() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  public void execute(Command command) {
    if (!command.canExecute()) {
      return;
    }

    redoList.clear();

    command.redoCommand();
    undoList.add(command);

    commandListenerProxy.fireCommandEvent( //
        new CommandEvent(this, command));
  }

  // --------------------------------------------------------------------------------

  public void undo() {
    if (undoList.isEmpty()) {
      return;
    }

    Command command = undoList.remove(ListUtils.end(undoList));
    command.undoCommand();
    redoList.add(command);

    commandListenerProxy.fireCommandEvent( //
        new CommandEvent(this, command));
  }

  // --------------------------------------------------------------------------------

  public void redo() {
    if (redoList.isEmpty()) {
      return;
    }

    Command command = redoList.remove(ListUtils.end(redoList));
    command.redoCommand();
    undoList.add(command);

    commandListenerProxy.fireCommandEvent( //
        new CommandEvent(this, command));
  }

  // --------------------------------------------------------------------------------

  public CommandListenerProxy getCommandListenerProxy() {
    return commandListenerProxy;
  }
}
