package org.cyrano.timetable.ui.command;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.tool.RectangleMouseHandler;
import org.cyrano.timetable.ui.model.hour.Hour;
import org.cyrano.timetable.ui.model.hour.HourBlock;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.utils.Rect;

public class SlotWHCommand implements Command {

  private String newBeg;
  private String newEnd;

  private String oldBeg;
  private String oldEnd;

  private Slot slot;

  // --------------------------------------------------------------------------------

  public SlotWHCommand(RectangleMouseHandler rmh, Slot slot) {
    this.slot = slot;

    oldBeg = slot.getBeg();
    oldEnd = slot.getEnd();

    calcNewParams(rmh);
  }

  // --------------------------------------------------------------------------------

  private void calcNewParams(RectangleMouseHandler rmh) {
    HourModel hourModel = //
    slot.getRoot().getHourModel();

    Rect rect = slot.getRect();

    int slotMidX = (rect.minX() + rect.maxX()) / 2;

    Hour dragTgtHour = hourModel.getHourAt(slotMidX, rmh.getClickUpPt().y);

    if (dragTgtHour == null) {
      return;
    }

    HourBlock hourBlock = hourModel.getHourBlock(slot);
    Hour dragBegHour = hourBlock.getBegHour();

    int begIndex = hourModel.getHourList().indexOf(dragBegHour);
    int tgtIndex = hourModel.getHourList().indexOf(dragTgtHour);

    if (begIndex <= tgtIndex) {
      newBeg = slot.getBeg();
      newEnd = dragTgtHour.end;
    } else {
      newBeg = dragTgtHour.beg;
      newEnd = dragBegHour.beg;
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  public void redoCommand() {
    slot.setBeg(newBeg);
    slot.setEnd(newEnd);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void undoCommand() {
    slot.setBeg(oldBeg);
    slot.setEnd(oldEnd);
  }

  // --------------------------------------------------------------------------------

  @Override
  public boolean canExecute() {
    //@begnf
    return (newBeg != null        &&  newEnd != null) && //
          (!newBeg.equals(oldBeg) || !newEnd.equals(oldEnd));
    //@endnf
  }
}
