package org.cyrano.timetable.ui.command;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.tool.RectangleMouseHandler;
import org.cyrano.timetable.ui.model.hour.Hour;
import org.cyrano.timetable.ui.model.hour.HourBlock;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.timetable.ui.model.timetable.Slot;

public class SlotXYCommand implements Command {

  private String newDay;
  private String newBeg;
  private String newEnd;

  private String oldDay;
  private String oldBeg;
  private String oldEnd;

  private Slot slot;

  // --------------------------------------------------------------------------------

  public SlotXYCommand(RectangleMouseHandler rmh, Slot slot) {
    this.slot = slot;

    oldDay = slot.getDay();
    oldBeg = slot.getBeg();
    oldEnd = slot.getEnd();

    calcNewParams(rmh);
  }

  // --------------------------------------------------------------------------------

  private void calcNewParams(RectangleMouseHandler rmh) {
    HourModel hourModel = //
    slot.getRoot().getHourModel();

    Hour dragSrcHour = hourModel.getHourAt(//
        rmh.getClickDwPt().x, rmh.getClickDwPt().y);

    Hour dragTgtHour = hourModel.getHourAt(//
        rmh.getClickUpPt().x, rmh.getClickUpPt().y);

    if (dragSrcHour == null || dragTgtHour == null) {
      return;
    }

    HourBlock srcHourBlock = hourModel.getHourBlock(slot);

    Hour dragBegHour = srcHourBlock.getBegHour();
    Hour dragEndHour = srcHourBlock.getEndHour();

    HourBlock tgtHourBlock = hourModel.getHourBlock( //
        dragBegHour, dragSrcHour, dragEndHour, dragTgtHour);

    if (tgtHourBlock == null) {
      return;
    }

    Hour newBegHour = tgtHourBlock.getBegHour();
    Hour newEndHour = tgtHourBlock.getEndHour();

    newDay = newBegHour.day;
    newBeg = newBegHour.beg;
    newEnd = newEndHour.end;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void redoCommand() {
    slot.setDay(newDay);
    slot.setBeg(newBeg);
    slot.setEnd(newEnd);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void undoCommand() {
    slot.setDay(oldDay);
    slot.setBeg(oldBeg);
    slot.setEnd(oldEnd);
  }

  // --------------------------------------------------------------------------------

  @Override
  public boolean canExecute() {
    //@begnf
    return (newDay != null        &&  newBeg != null        &&  newEnd != null) && //
          (!newDay.equals(oldDay) || !newBeg.equals(oldBeg) || !newEnd.equals(oldEnd));
    //@endnf
  }
}
