package org.cyrano.timetable.ui.edit;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

public abstract class DlgEditBase<T> extends JDialog {

  protected T data;

  // --------------------------------------------------------------------------------

  public DlgEditBase(JFrame parent, T data, //
      String createTitle, String updateTitle) {
    this.data = data;

    setTitle(data == null ? createTitle : updateTitle);
    setModal(true);
    setResizable(false);

    initGUI();

    pack();

    setLocationRelativeTo(parent);
  }

  // --------------------------------------------------------------------------------

  protected GridBagConstraints initGBC( //
      int gx, int gy, int wx, int wy, int gw, int gh) {

    GridBagConstraints ret = new GridBagConstraints();

    ret.gridx/*     */= gx;
    ret.gridy/*     */= gy;
    ret.weightx/*   */= wx;
    ret.weighty/*   */= wy;
    ret.gridwidth/* */= gw;
    ret.gridheight/**/= gh;

    return ret;
  }

  // --------------------------------------------------------------------------------

  protected void initGUI() {
    setLayout(new BorderLayout());

    add(initUpdatePanel(), BorderLayout.CENTER);
    add(initButtonPanel(), BorderLayout.SOUTH);
  }

  // --------------------------------------------------------------------------------

  private JPanel initButtonPanel() {
    JPanel ret = new JPanel(new FlowLayout());

    JButton btnAccept = new JButton(_I18NDlgEditBase.accept());
    btnAccept.setMinimumSize(new Dimension(80, 30));
    btnAccept.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent evt) {
        if (!validateData()) {
          return;
        }

        if (data == null) {
          createBtnAcceptClicked();
        } else {
          updateBtnAcceptClicked();
        }
      }
    });
    ret.add(btnAccept);

    JButton btnCancel = new JButton(_I18NDlgEditBase.cancel());
    btnCancel.setMinimumSize(new Dimension(80, 30));
    btnCancel.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent evt) {
        dispose();
      }
    });
    ret.add(btnCancel);

    return ret;
  }

  // --------------------------------------------------------------------------------

  protected boolean validateData() {
    return true;
  }

  // --------------------------------------------------------------------------------

  protected abstract JPanel initUpdatePanel();

  // --------------------------------------------------------------------------------

  protected abstract void createBtnAcceptClicked();

  protected abstract void updateBtnAcceptClicked();
}
