package org.cyrano.timetable.ui.edit.cour;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.ComposedCommand;
import org.cyrano.timetable.ui.command.ComposedCommand.CanExecuteMode;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.command.ReparentCommand;
import org.cyrano.timetable.ui.command.UpdateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.LabelGetter;
import org.cyrano.timetable.ui.edit.LabelValueBeanComboBoxModel;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sems;

import com.minotauro.cleda.util.LabelValueBean;

public class DlgEditCour extends DlgEditBase<Cour> {

  private Sems sems;

  private JComboBox<LabelValueBean<Sems>> cboSems;

  private JTextField txtName;

  // --------------------------------------------------------------------------------

  public DlgEditCour(JFrame parent, Cour data, Sems sems) {
    super(parent, data, //
        _I18NDlgEditCour.createTitle(), //
        _I18NDlgEditCour.updateTitle());

    this.sems = sems;

    guiFromData(data);
    initCboSems();
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Cour cour) {
    cour.setName(txtName.getText());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Cour cour) {
    if (cour == null) {
      return;
    }

    txtName.setText(cour.getName());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditCour.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    JLabel lblSems = new JLabel(_I18NDlgEditCour.sems());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(5, 5, 0, 0);
    ret.add(lblSems, gbc);

    cboSems = new JComboBox<LabelValueBean<Sems>>();
    cboSems.setModel(new LabelValueBeanComboBoxModel<Sems>());
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(5, 5, 0, 5);
    ret.add(cboSems, gbc);

    // ----------------------------------------

    row++;

    JLabel lblName = new JLabel(_I18NDlgEditCour.name());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblName, gbc);

    txtName = new JTextField();
    txtName.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(txtName, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  private void initCboSems() {
    Root timeTable = data != null ? data.getRoot() : sems.getRoot();

    LabelValueBeanComboBoxModel<Sems> dcbm = //
    (LabelValueBeanComboBoxModel<Sems>) cboSems.getModel();
    dcbm.removeAllElements();

    dcbm.addAll(timeTable.getSemsList().getChildrenList(), new LabelGetter<Sems>() {
      @Override
      public String getLabel(Sems data) {
        return data.getName();
      }
    });

    Sems selected = sems != null //
        ? sems
        : data.getSemsRef().getParentRef();

    dcbm.setSelectedData(selected);
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    if (StringUtils.isBlank(txtName.getText())) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditCour.errorEmptyName(), //
          _I18NDlgEditCour.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    LabelValueBeanComboBoxModel<Sems> dcbm = //
    (LabelValueBeanComboBoxModel<Sems>) cboSems.getModel();

    data = new Cour();
    dataFromGui(data);

    Command command = //
    new CreateChildCommand<Sems, Cour>( //
        dcbm.getSelectedData().getCourList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    LabelValueBeanComboBoxModel<Sems> dcbm = //
    (LabelValueBeanComboBoxModel<Sems>) cboSems.getModel();

    Cour prevCour = new Cour();
    prevCour.shallowDataCopyFrom(data);

    Cour nextCour = new Cour();
    dataFromGui(nextCour);

    Command commandUpdate = new UpdateChildCommand( //
        prevCour, data, nextCour);

    Command commandChange = //
    new ReparentCommand<Sems, Cour>(data, //
        data.getSemsRef().getParentRef().getCourList(), //
        /*      */dcbm.getSelectedData().getCourList());

    ComposedCommand composedCommand = //
    new ComposedCommand(CanExecuteMode.ANY);

    composedCommand.add(commandUpdate);
    composedCommand.add(commandChange);

    UndoRedoManager.getInstance().execute(composedCommand);

    dispose();
  }
}
