package org.cyrano.timetable.ui.edit.prof;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.command.UpdateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.JColorButton;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Root;

public class DlgEditProf extends DlgEditBase<Prof> {

  private Root root;

  private JTextField txtInit;

  private JColorButton btnBackground;

  // --------------------------------------------------------------------------------

  public DlgEditProf(JFrame parent, Prof data, Root root) {
    super(parent, data, //
        _I18NDlgEditProf.createTitle(), //
        _I18NDlgEditProf.updateTitle());

    this.root = root;

    guiFromData(data);
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Prof data) {
    data.setInit(txtInit.getText());
    data.setBackground(btnBackground.getColor());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Prof data) {
    if (data == null) {
      return;
    }

    txtInit.setText(data.getInit());
    btnBackground.setColor(data.getBackground());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditProf.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    // ----------------------------------------

    JLabel lblInit = new JLabel(_I18NDlgEditProf.init());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblInit, gbc);

    txtInit = new JTextField();
    txtInit.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(txtInit, gbc);

    // ----------------------------------------

    row++;

    JLabel lblBackground = new JLabel(_I18NDlgEditProf.background());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblBackground, gbc);

    btnBackground = new JColorButton();
    gbc = initGBC(1, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.NONE;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(btnBackground, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    if (StringUtils.isBlank(txtInit.getText())) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditProf.errorEmptyInit(), //
          _I18NDlgEditProf.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    data = new Prof();
    dataFromGui(data);

    Command command = //
    new CreateChildCommand<Root, Prof>( //
        root.getProfList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    Prof prevProf = new Prof();
    prevProf.shallowDataCopyFrom(data);

    Prof nextProf = new Prof();
    dataFromGui(nextProf);

    Command command = new UpdateChildCommand( //
        prevProf, data, nextProf);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }
}
