package org.cyrano.timetable.ui.edit.room;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.command.UpdateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.JColorButton;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;

public class DlgEditRoom extends DlgEditBase<Room> {

  private Root root;

  private JTextField txtName;

  private JColorButton btnBackground;

  // --------------------------------------------------------------------------------

  public DlgEditRoom(JFrame parent, Room data, Root root) {
    super(parent, data, //
        _I18NDlgEditRoom.createTitle(), //
        _I18NDlgEditRoom.updateTitle());

    this.root = root;

    guiFromData(data);
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Room data) {
    data.setName(txtName.getText());
    data.setBackground(btnBackground.getColor());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Room data) {
    if (data == null) {
      return;
    }

    txtName.setText(data.getName());
    btnBackground.setColor(data.getBackground());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditRoom.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    // ----------------------------------------

    JLabel lblName = new JLabel(_I18NDlgEditRoom.name());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblName, gbc);

    txtName = new JTextField();
    txtName.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(txtName, gbc);

    // ----------------------------------------

    row++;

    JLabel lblBackground = new JLabel(_I18NDlgEditRoom.background());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblBackground, gbc);

    btnBackground = new JColorButton();
    gbc = initGBC(1, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.NONE;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(btnBackground, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    if (StringUtils.isBlank(txtName.getText())) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditRoom.errorEmptyInit(), //
          _I18NDlgEditRoom.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    data = new Room();
    dataFromGui(data);

    Command command = //
    new CreateChildCommand<Root, Room>( //
        root.getRoomList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    Room prevRoom = new Room();
    prevRoom.shallowDataCopyFrom(data);

    Room nextRoom = new Room();
    dataFromGui(nextRoom);

    Command command = new UpdateChildCommand( //
        prevRoom, data, nextRoom);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }
}
