package org.cyrano.timetable.ui.edit.sems;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.DeleteChildCommand;
import org.cyrano.timetable.ui.edit.BaseTreePanel;
import org.cyrano.timetable.ui.edit.cour.DlgEditCour;
import org.cyrano.timetable.ui.edit.sect.DlgEditSect;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.DisplayMode;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.utils.ImageCache;

public class SemsTreePanel extends BaseTreePanel {

  private Action actCreateSems;
  private Action actUpdateSems;
  private Action actDeleteSems;

  private Action actCreateCour;
  private Action actUpdateCour;
  private Action actDeleteCour;

  private Action actCreateSect;
  private Action actUpdateSect;
  private Action actDeleteSect;

  // --------------------------------------------------------------------------------

  public SemsTreePanel() {
    super(DisplayMode.SEMS);

    initActions();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void initActions() {
    actCreateSems = new AbstractAction(//
        _I18NSemsTreePanel.createSems(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_create.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actCreateSemsClicked();
      }
    };

    actUpdateSems = new AbstractAction(//
        _I18NSemsTreePanel.updateSems(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actUpdateSemsClicked();
      }
    };

    actDeleteSems = new AbstractAction(//
        _I18NSemsTreePanel.deleteSems(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actDeleteSemsClicked();
      }
    };

    actCreateCour = new AbstractAction(//
        _I18NSemsTreePanel.createCour(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_create.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actCreateCourClicked();
      }
    };

    actUpdateCour = new AbstractAction(//
        _I18NSemsTreePanel.updateCour(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actUpdateCourClicked();
      }
    };

    actDeleteCour = new AbstractAction(//
        _I18NSemsTreePanel.deleteCour(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actDeleteCourClicked();
      }
    };

    actCreateSect = new AbstractAction(//
        _I18NSemsTreePanel.createSect(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_create.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actCreateSectClicked();
      }
    };

    actUpdateSect = new AbstractAction(//
        _I18NSemsTreePanel.updateSect(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actUpdateSectClicked();
      }
    };

    actDeleteSect = new AbstractAction(//
        _I18NSemsTreePanel.deleteSect(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actDeleteSectClicked();
      }
    };
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnCreateClicked() {
    BaseModel baseModel = getSelectedObject();

    /*   */if (baseModel instanceof Root) {
      actCreateSems.actionPerformed(null);
    } else if (baseModel instanceof Sems) {
      actCreateCour.actionPerformed(null);
    } else if (baseModel instanceof Cour) {
      actCreateSect.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnUpdateClicked() {
    BaseModel baseModel = getSelectedObject();

    /*   */if (baseModel instanceof Sems) {
      actUpdateSems.actionPerformed(null);
    } else if (baseModel instanceof Cour) {
      actUpdateCour.actionPerformed(null);
    } else if (baseModel instanceof Sect) {
      actUpdateSect.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnDeleteClicked() {
    BaseModel baseModel = getSelectedObject();

    /*   */if (baseModel instanceof Sems) {
      actDeleteSems.actionPerformed(null);
    } else if (baseModel instanceof Cour) {
      actDeleteCour.actionPerformed(null);
    } else if (baseModel instanceof Sect) {
      actDeleteSect.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPopupMenu createPopupMenuFor(Object object) {
    if (object instanceof Root) {
      return createPopupMenuForRoot();
    }

    if (object instanceof Sems) {
      return createPopupMenuForSems();
    }

    if (object instanceof Cour) {
      return createPopupMenuForCour();
    }

    if (object instanceof Sect) {
      return createPopupMenuForSect();
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForRoot() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actCreateSems);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForSems() {
    JPopupMenu ret = new JPopupMenu();

    ret.add(actUpdateSems);
    ret.add(actDeleteSems);
    ret.add(actCreateCour);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForCour() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actUpdateCour);
    ret.add(actDeleteCour);
    ret.add(actCreateSect);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForSect() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actUpdateSect);
    ret.add(actDeleteSect);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private void actCreateSemsClicked() {
    Root root = (Root) getSelectedObject();
    DlgEditSems edt = new DlgEditSems( //
        MainFrame.workbenchRef, null, root);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actUpdateSemsClicked() {
    Sems sems = (Sems) getSelectedObject();
    DlgEditSems edt = new DlgEditSems( //
        MainFrame.workbenchRef, sems, null);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actDeleteSemsClicked() {
    Sems sems = (Sems) getSelectedObject();
    DeleteChildCommand<Root, Sems> command = //
    new DeleteChildCommand<Root, Sems>( //
        sems.getRootRef().getParentRef().getSemsList(), sems);
    UndoRedoManager.getInstance().execute(command);
  }

  // --------------------------------------------------------------------------------

  private void actCreateCourClicked() {
    Sems sems = (Sems) getSelectedObject();
    DlgEditCour edt = new DlgEditCour( //
        MainFrame.workbenchRef, null, sems);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actUpdateCourClicked() {
    Cour cour = (Cour) getSelectedObject();
    DlgEditCour edt = new DlgEditCour( //
        MainFrame.workbenchRef, cour, null);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actDeleteCourClicked() {
    Cour cour = (Cour) getSelectedObject();
    DeleteChildCommand<Sems, Cour> command = //
    new DeleteChildCommand<Sems, Cour>( //
        cour.getSemsRef().getParentRef().getCourList(), cour);
    UndoRedoManager.getInstance().execute(command);
  }

  // --------------------------------------------------------------------------------

  private void actCreateSectClicked() {
    Cour cour = (Cour) getSelectedObject();
    DlgEditSect edt = new DlgEditSect( //
        MainFrame.workbenchRef, null, cour);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actUpdateSectClicked() {
    Sect sect = (Sect) getSelectedObject();
    DlgEditSect edt = new DlgEditSect( //
        MainFrame.workbenchRef, sect, null);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void actDeleteSectClicked() {
    Sect sect = (Sect) getSelectedObject();
    DeleteChildCommand<Cour, Sect> command = //
    new DeleteChildCommand<Cour, Sect>( //
        sect.getCourRef().getParentRef().getSectList(), sect);
    UndoRedoManager.getInstance().execute(command);
  }
}
