package org.cyrano.timetable.ui.edit.slot;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.LabelGetter;
import org.cyrano.timetable.ui.edit.LabelValueBeanComboBoxModel;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;

import com.minotauro.cleda.util.LabelValueBean;

public class DlgCreateSlot extends DlgEditBase<Slot> {

  private JComboBox<LabelValueBean<Room>> cboRoom;
  private JComboBox<LabelValueBean<Cour>> cboCour;
  private JComboBox<LabelValueBean<Sect>> cboSect;

  private ActionListener cboCourActionListener;

  private String day;
  private String beg;
  private String end;

  private Root root;

  // --------------------------------------------------------------------------------

  public DlgCreateSlot(JFrame parent, Root root, //
      String day, String beg, String end) {
    super(parent, null, //
        _I18NDlgEditSlot.createTitle(), //
        _I18NDlgEditSlot.updateTitle());

    this.root = root;

    this.day = day;
    this.beg = beg;
    this.end = end;

    initCboCour();
    initCboSect();
    initCboRoom();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditSlot.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    JLabel lblCour = new JLabel(_I18NDlgEditSlot.cour());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 0);
    ret.add(lblCour, gbc);

    cboCour = new JComboBox<LabelValueBean<Cour>>();
    cboCour.setModel(new LabelValueBeanComboBoxModel<Cour>());
    cboCour.setPreferredSize(new Dimension(200, 25));

    cboCourActionListener = new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        initCboSect();
      }
    };
    cboCour.addActionListener(cboCourActionListener);

    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 5);
    ret.add(cboCour, gbc);

    // ----------------------------------------

    row++;

    JLabel lblSect = new JLabel(_I18NDlgEditSlot.sect());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 0);
    ret.add(lblSect, gbc);

    cboSect = new JComboBox<LabelValueBean<Sect>>();
    cboSect.setModel(new LabelValueBeanComboBoxModel<Sect>());
    cboSect.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 5);
    ret.add(cboSect, gbc);

    // ----------------------------------------

    row++;

    JLabel lblRoom = new JLabel(_I18NDlgEditSlot.room());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblRoom, gbc);

    cboRoom = new JComboBox<LabelValueBean<Room>>();
    cboRoom.setModel(new LabelValueBeanComboBoxModel<Room>());
    cboRoom.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(cboRoom, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  private void initCboCour() {
    LabelValueBeanComboBoxModel<Cour> dcbmCour = //
    (LabelValueBeanComboBoxModel<Cour>) cboCour.getModel();
    dcbmCour.removeAllElements();

    List<LabelValueBean<Cour>> courList = //
    new ArrayList<LabelValueBean<Cour>>();

    for (Sems sems : root.getSemsList().getChildrenList()) {
      for (Cour cour : sems.getCourList().getChildrenList()) {
        courList.add(new LabelValueBean<Cour>(cour.getName(), cour));
      }
    }

    Collections.sort(courList, new Comparator<LabelValueBean<Cour>>() {
      @Override
      public int compare( //
          LabelValueBean<Cour> lvb1, //
          LabelValueBean<Cour> lvb2) {
        return lvb1.getLabel().compareTo(lvb2.getLabel());
      }
    });

    cboCour.removeActionListener(cboCourActionListener);

    dcbmCour.add(_I18NDlgEditSlot.select(), null);

    for (LabelValueBean<Cour> labelValueBean : courList) {
      dcbmCour.addElement(labelValueBean);
    }

    cboCour.addActionListener(cboCourActionListener);
  }

  // --------------------------------------------------------------------------------

  private void initCboSect() {
    LabelValueBeanComboBoxModel<Cour> dcbmCour = //
    (LabelValueBeanComboBoxModel<Cour>) cboCour.getModel();

    LabelValueBeanComboBoxModel<Sect> dcbmSect = //
    (LabelValueBeanComboBoxModel<Sect>) cboSect.getModel();
    dcbmSect.removeAllElements();

    List<LabelValueBean<Sect>> sectList = //
    new ArrayList<LabelValueBean<Sect>>();

    Cour cour = dcbmCour.getSelectedData();

    if (cour != null) {
      for (Sect sect : cour.getSectList().getChildrenList()) {
        sectList.add(new LabelValueBean<Sect>(sect.getNumb(), sect));
      }
    }

    Collections.sort(sectList, new Comparator<LabelValueBean<Sect>>() {
      @Override
      public int compare( //
          LabelValueBean<Sect> lvb1, //
          LabelValueBean<Sect> lvb2) {
        return lvb1.getLabel().compareTo(lvb2.getLabel());
      }
    });

    dcbmSect.add(_I18NDlgEditSlot.select(), null);

    for (LabelValueBean<Sect> labelValueBean : sectList) {
      dcbmSect.addElement(labelValueBean);
    }
  }

  // --------------------------------------------------------------------------------

  private void initCboRoom() {
    LabelValueBeanComboBoxModel<Room> dcbm = //
    (LabelValueBeanComboBoxModel<Room>) cboRoom.getModel();
    dcbm.removeAllElements();

    dcbm.add(_I18NDlgEditSlot.select(), null);

    dcbm.addAll(root.getRoomList().getChildrenList(), //
        new LabelGetter<Room>() {
          @Override
          public String getLabel(Room data) {
            return data.getName();
          }
        });
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    LabelValueBeanComboBoxModel<Cour> dcbmCour = //
    (LabelValueBeanComboBoxModel<Cour>) cboCour.getModel();
    Cour cour = dcbmCour.getSelectedData();

    if (cour == null) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditSlot.errorEmptyCour(), //
          _I18NDlgEditSlot.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    LabelValueBeanComboBoxModel<Sect> dcbmSect = //
    (LabelValueBeanComboBoxModel<Sect>) cboSect.getModel();
    Sect sect = dcbmSect.getSelectedData();

    if (sect == null) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditSlot.errorEmptySect(), //
          _I18NDlgEditSlot.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    LabelValueBeanComboBoxModel<Sect> dcbmSect = //
    (LabelValueBeanComboBoxModel<Sect>) cboSect.getModel();

    Sect sect = dcbmSect.getSelectedData();

    LabelValueBeanComboBoxModel<Room> dcbmRoom = //
    (LabelValueBeanComboBoxModel<Room>) cboRoom.getModel();

    Room room = dcbmRoom.getSelectedData();

    data = new Slot(day, beg, end, room, "lec");

    Command command = //
    new CreateChildCommand<Sect, Slot>( //
        sect.getSlotList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    throw new UnsupportedOperationException();
  }
}
