package org.cyrano.timetable.ui.figures;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;

import org.cyrano.editor.parts.BaseContainerViewPart;
import org.cyrano.editor.parts.PartsRegister;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.utils.Rect;

public class TimeTableViewPart extends BaseContainerViewPart {

  private static final Font LEGEND_FONT = //
  new Font("Arial", Font.BOLD, 20);

  public static final int LEGEND_XOFFSET = 100;
  public static final int LEGEND_YOFFSET = 30;

  private Rect rect;

  // --------------------------------------------------------------------------------

  public TimeTableViewPart(PartsRegister partsRegister) {
    super(partsRegister);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void paintLeaf(Graphics2D g2d) {
    g2d.setBackground(Color.WHITE);
    g2d.clearRect(0, 0, rect.w, rect.h);

    g2d.setColor(Color.BLACK);

    Font prevFont = g2d.getFont();

    g2d.setFont(LEGEND_FONT);

    Root timeTable = //
    (Root) getPartsRegister().getModelByViewPart(this);

    String[] cols = timeTable.getHourModel().getColsText();
    String[] rows = timeTable.getHourModel().getRowsText();

    int colW = timeTable.getHourModel().getColW();
    int rowH = timeTable.getHourModel().getRowH();

    FontMetrics fontMetrics = g2d.getFontMetrics(g2d.getFont());

    // ----------------------------------------
    // Rows
    // ----------------------------------------

    for (int i = 0, x = LEGEND_XOFFSET; i < cols.length; i++, x += colW) {
      g2d.drawLine(x, 0, x, rect.h);

      int strW = fontMetrics.stringWidth(cols[i]);
      int strH = fontMetrics.getDescent() + fontMetrics.getAscent();
      int strY = (LEGEND_YOFFSET + strH) / 2 - fontMetrics.getDescent();
      int strX = x + (colW - strW) / 2;
      g2d.drawString(cols[i], strX, strY);
    }

    // ----------------------------------------
    // Cols
    // ----------------------------------------

    for (int i = 0, y = LEGEND_YOFFSET; i < rows.length; i++, y += rowH) {
      g2d.drawLine(0, y, rect.w, y);

      int strW = fontMetrics.stringWidth(rows[i]);
      int strH = fontMetrics.getDescent() + fontMetrics.getAscent();
      int strY = y + (rowH + strH) / 2 - fontMetrics.getDescent();
      int strX = (LEGEND_XOFFSET - strW) / 2;
      g2d.drawString(rows[i], strX, strY);
    }

    g2d.setFont(prevFont);
  }

  // --------------------------------------------------------------------------------

  @Override
  public Rect getRect() {
    return rect;
  }

  @Override
  public void setRect(Rect rect) {
    this.rect = rect;
  }
}
