package org.cyrano.timetable.ui.model.base;

import javax.swing.event.EventListenerList;

import org.cyrano.timetable.ui.model.base.StructureChangeListener.StructureChangeMethod;

public class StructureChangeSupport {

  private final EventListenerList eventListenerList;

  // --------------------------------------------------------------------------------

  public StructureChangeSupport(EventListenerList eventListenerList) {
    this.eventListenerList = eventListenerList;
  }

  // --------------------------------------------------------------------------------

  public void addStructureChangeListener(
      StructureChangeListener structureChangeListener) {

    eventListenerList.add( //
        StructureChangeListener.class, structureChangeListener);
  }

  // --------------------------------------------------------------------------------

  public void delStructureChangeListener(
      StructureChangeListener structureChangeListener) {

    eventListenerList.remove( //
        StructureChangeListener.class, structureChangeListener);
  }

  // --------------------------------------------------------------------------------

  public StructureChangeListener[] getStructureChangeListeners() {
    return eventListenerList.getListeners( //
        StructureChangeListener.class);
  }

  // --------------------------------------------------------------------------------

  public final void fireStructureChange( //
      StructureChangeEvent evt, //
      StructureChangeMethod method) {

    StructureChangeListener[] listeners = //
    getStructureChangeListeners();

    for (int i = 0; i < listeners.length; i++) {
      switch (method) {
        case ADDCHILD :
          listeners[i].addChild(evt);
          break;
        case DELCHILD :
          listeners[i].delChild(evt);
          break;
        case REPARENT :
          listeners[i].reparent(evt);
          break;
        default :
          throw new IllegalArgumentException( //
              method.toString());
      }
    }
  }
}
