package org.cyrano.timetable.ui.model.timetable;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.base.ModelChangeEvent;
import org.cyrano.timetable.ui.model.base.ObjectMatcher;
import org.cyrano.timetable.ui.model.base.ObjectVisitor;
import org.cyrano.timetable.ui.model.base.Parent;
import org.cyrano.timetable.ui.model.base.ParentChildrenImpl;
import org.cyrano.timetable.ui.model.base.Setter;

public class Cour implements BaseModel {

  public static final ObjectMatcher<Cour, String> COUR_BY_NAME_MATCHER = //
  new ObjectMatcher<Cour, String>() {
    @Override
    public boolean matches(Cour object, String target) {
      return target.equals(object.getName());
    }
  };

  // --------------------------------------------------------------------------------

  private Children<Sems> semsRef;

  private Parent<Cour, Sect> sectList;

  private String name;

  // --------------------------------------------------------------------------------

  public Cour() {
    ParentChildrenImpl<Cour, Sect> sectList = //
    new ParentChildrenImpl<Cour, Sect>(this);
    sectList.setSetter(new Setter<Sect, Children<Cour>>() {
      @Override
      public void set(Sect target, Children<Cour> value) {
        target.setCourRef(value);
      }
    });
    sectList.setComparator(new Comparator<Sect>() {
      @Override
      public int compare(Sect s1, Sect s2) {
        return s1.getDisplayName().compareTo(s2.getDisplayName());
      }
    });
    this.sectList = sectList;
  }

  // --------------------------------------------------------------------------------

  public Children<Sems> getSemsRef() {
    return semsRef;
  }

  public void setSemsRef(Children<Sems> semsRef) {
    this.semsRef = semsRef;

    if (semsRef != null) {
      sectList.setStructureChangeSupport( //
          getRoot().getStructureChangeSupport());
    } else {
      sectList.setStructureChangeSupport(null);
    }
  }

  // --------------------------------------------------------------------------------

  public Parent<Cour, Sect> getSectList() {
    return sectList;
  }

  // --------------------------------------------------------------------------------

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  // --------------------------------------------------------------------------------

  public Sect findSectByNumb(String numb) {
    return sectList.findSingChildBy( //
        Sect.SECT_BY_NUMB_MATCHER, numb);
  }

  // --------------------------------------------------------------------------------

  public void clearValidations() {
    for (Sect sect : sectList.getChildrenList()) {
      sect.clearValidations();
    }
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return semsRef != null ? semsRef.getParentRef().getRoot() : null;
  }

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    return getName();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setVisible(final boolean visible) {
    sectList.visitChildren(//
        new ObjectVisitor<Sect>() {
          @Override
          public void visit(Sect object) {
            object.setVisible(visible);
          }
        });
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<Slot> getAllChildrenSlots() {
    List<Slot> ret = new ArrayList<Slot>();

    for (Sect sect : sectList.getChildrenList()) {
      ret.addAll(sect.getAllChildrenSlots());
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    if (!(baseModel instanceof Cour)) {
      throw new IllegalArgumentException();
    }

    Cour from = (Cour) baseModel;

    this.name = from.name;

    if (getRoot() != null) {
      getRoot().getModelChangeSupport().fireModelChange(new ModelChangeEvent(this, this));
    }
  }
}
